MODULE Soundex;
(* =========================================================================  
   Soundex - Example RISC5 Oberon Program  
 
   Description:
     Compare the performance of numeric CASE vs. IF-THEN-ELSIF statements
     
   (c) 2018 CFB Software   
   http://www.astrobe.com  

  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

IMPORT 
  Oberon, Out;

CONST
  iterations = 100000;

PROCEDURE SoundexIf(ch: CHAR): INTEGER;
VAR 
  value: INTEGER;
BEGIN
  IF (ch = "A") OR (ch = "E") OR (ch = "I") OR (ch = "O") OR (ch = "U") OR (ch = "H") OR (ch = "W") OR (ch = "Y") THEN
    value := 0 
  ELSIF (ch = "B") OR (ch = "F") OR (ch = "P") OR (ch = "V") THEN
    value := 1 
  ELSIF (ch = "C") OR (ch = "G") OR (ch = "J") OR (ch = "K") OR (ch = "Q") OR (ch = "S") OR (ch = "X") OR (ch = "Z") THEN 
    value := 2 
  ELSIF (ch = "D") OR (ch = "T") THEN
    value := 3 
  ELSIF (ch = "L") THEN
    value := 4 
  ELSIF (ch = "M") OR (ch = "N") THEN
    value := 5 
  ELSIF (ch = "R") THEN
    value := 6
  END; 
  RETURN value
END SoundexIf;

PROCEDURE SoundexCase(ch: CHAR): INTEGER;
VAR 
  value: INTEGER;
BEGIN
  CASE ch OF
   "A", "E", "I", "O", "U", "H", "W", "Y":
      value := 0 |
   "B", "F", "P", "V":
      value := 1 |
   "C", "G", "J", "K", "Q", "S", "X", "Z":
      value := 2 |
   "D", "T":
      value := 3 |
   "L":
      value := 4 |
   "M", "N":
      value := 5 |
   "R":
      value := 6 
  END;
  RETURN value
END SoundexCase;

PROCEDURE Run*;
VAR
  ch, i, value, start, elapsed: INTEGER;
BEGIN
  Out.String("Soundex"); Out.Ln;
  FOR ch := ORD("A") TO ORD("Z") DO
    Out.Char(CHR(ch)); 
    start := Oberon.Time();
    FOR i := 1 TO iterations DO 
      value := SoundexIf(CHR(ch));
    END;
    elapsed := Oberon.Time() - start;
    Out.Int(elapsed, 5);
    start := Oberon.Time();
    FOR i := 1 TO iterations DO 
      value := SoundexCase(CHR(ch));
    END;
    elapsed := Oberon.Time() - start;
    Out.Int(elapsed, 5); Out.String(" Msec"); Out.Ln();
  END;
END Run;

END Soundex.
